@@ -1,4 +1,8 @@
-{-# LANGUAGE FlexibleContexts, FlexibleInstances, GADTs, RankNTypes #-}
+{-# LANGUAGE FlexibleContexts #-}
+{-# LANGUAGE FlexibleInstances #-}
+{-# LANGUAGE GADTs #-}
+{-# LANGUAGE RankNTypes #-}
+
 -- | The @esqueleto@ EDSL (embedded domain specific language).
 -- This module replaces @Database.Persist@, so instead of
 -- importing that module you should just import this one:
@@ -27,7 +31,16 @@
 --
 -- Other than identifier name clashes, @esqueleto@ does not
 -- conflict with @persistent@ in any way.
-module Database.Esqueleto
+--
+-- Note that the facilities for @JOIN@ have been significantly improved in the
+-- "Database.Esqueleto.Experimental" module. The definition of 'from' and 'on'
+-- in this module will be replaced with those at the 4.0.0.0 version, so you are
+-- encouraged to migrate to the new method.
+--
+-- This module has an attached WARNING message indicating that the Experimental
+-- syntax will become the default. If you want to continue using the old syntax,
+-- please refer to "Database.Esqueleto.Legacy" as a drop-in replacement.
+module Database.Esqueleto {-# WARNING "This module will switch over to the Experimental syntax in an upcoming major version release. Please migrate to the Database.Esqueleto.Legacy module to continue using the old syntax, or translate to the new and improved syntax in Database.Esqueleto.Experimental." #-}
   ( -- * Setup
     -- $setup
 
@@ -84,6 +97,7 @@ module Database.Esqueleto
   , SqlExpr
   , SqlEntity
   , select
+  , selectOne
   , selectSource
   , delete
   , deleteCount
@@ -115,14 +129,8 @@ module Database.Esqueleto
   , module Database.Esqueleto.Internal.PersistentImport
   ) where
 
-import Control.Monad.IO.Class (MonadIO)
-import Control.Monad.Trans.Reader (ReaderT)
-import Data.Int (Int64)
-import qualified Data.Map.Strict as Map
-import Database.Esqueleto.Internal.Language
-import Database.Esqueleto.Internal.Sql
+import Database.Esqueleto.Legacy
 import Database.Esqueleto.Internal.PersistentImport
-import qualified Database.Persist
 
 
 -- $setup
@@ -405,66 +413,3 @@ import qualified Database.Persist
 --
 -- In order to use these functions, you need to explicitly import
 -- their corresponding modules, they're not re-exported here.
-
-
-----------------------------------------------------------------------
-
-
--- | @valkey i = 'val' . 'toSqlKey'@
--- (<https://github.com/prowdsponsor/esqueleto/issues/9>).
-valkey :: (ToBackendKey SqlBackend entity, PersistField (Key entity)) =>
-          Int64 -> SqlExpr (Value (Key entity))
-valkey = val . toSqlKey
-
-
--- | @valJ@ is like @val@ but for something that is already a @Value@. The use
--- case it was written for was, given a @Value@ lift the @Key@ for that @Value@
--- into the query expression in a type safe way. However, the implementation is
--- more generic than that so we call it @valJ@.
---
--- Its important to note that the input entity and the output entity are
--- constrained to be the same by the type signature on the function
--- (<https://github.com/prowdsponsor/esqueleto/pull/69>).
---
--- /Since: 1.4.2/
-valJ :: (PersistField (Key entity)) =>
-        Value (Key entity) -> SqlExpr (Value (Key entity))
-valJ = val . unValue
-
-
-----------------------------------------------------------------------
-
-
--- | Synonym for 'Database.Persist.Store.delete' that does not
--- clash with @esqueleto@'s 'delete'.
-deleteKey :: ( PersistStore backend
-             , BaseBackend backend ~ PersistEntityBackend val
-             , MonadIO m
-             , PersistEntity val )
-          => Key val -> ReaderT backend m ()
-deleteKey = Database.Persist.delete
-
--- | Avoid N+1 queries and join entities into a map structure
--- @
--- getFoosAndNestedBarsFromParent :: ParentId -> (Map (Key Foo) (Foo, [Maybe (Entity Bar)]))
--- getFoosAndNestedBarsFromParent parentId = 'fmap' associateJoin $ 'select' $
--- 'from' $ \\(foo `'LeftOuterJoin`` bar) -> do
---   'on' (bar '?.' BarFooId '==.' foo '^.' FooId)
---   'where_' (foo '^.' FooParentId '==.' 'val' parentId)
---   'pure' (foo, bar)
--- @
---
--- @since 3.1.2
-associateJoin
-  :: forall e1 e0
-   . Ord (Key e0)
-  => [(Entity e0, e1)]
-  -> Map.Map (Key e0) (e0, [e1])
-associateJoin = foldr f start
-  where
-    start = Map.empty
-    f (one, many) =
-      Map.insertWith
-        (\(oneOld, manyOld) (_, manyNew) -> (oneOld, manyNew ++ manyOld ))
-        (entityKey one)
-        (entityVal one, [many])
